

#include <stdlib.h>
#include "MIDIShuffler.H"


using namespace std;
using namespace avelino;


int32_t MIDIShuffler::getDelayedInstant(int32_t sourceInstant) {
	return ((this->sixteenthNoteLength - this->reducedLength) + ((sourceInstant * this->reducedLength) / this->sixteenthNoteLength));
}


void MIDIShuffler::byteReceived(uint8_t byte) {
	this->rxQueue.push(byte);
}


void MIDIShuffler::processRxByte(uint8_t byte) {
	bool send = true;
	uint8_t noChannelByte = byte & 0xF0;
	if (this->status == STATUS_WAIT_START_MIDI_CLOCK) {
		if (byte == 0xFA) {
			this->delayQueue.clear();
			this->rxQueue.clear();
			this->status = STATUS_WAIT_FIRST_MIDI_CLOCK;
		}
	}
	else if (this->status == STATUS_WAIT_FIRST_MIDI_CLOCK) {
		if (byte == 0xF8) {
			this->status = STATUS_WAIT_MIDI_CLOCK;
			this->clockCounter = CLOCK_PER_SIXTEENTH_NOTE;
			this->sixteenthNoteIndex = 0;
			this->timeCounter->start();
		}
	}
	else if (this->status == STATUS_WAIT_MIDI_CLOCK) {
		if (byte == 0xF8) {
			this->clockCounter--;
			if (this->clockCounter == 0) {
				if ((this->sixteenthNoteIndex == 0) || (this->sixteenthNoteIndex == 2)) {
					this->sixteenthNoteLength = this->timeCounter->getValue();
					this->reducedLength = (this->sixteenthNoteLength * (100 - this->percentProvider->getPercent())) / 100;
				}
				this->clockCounter = CLOCK_PER_SIXTEENTH_NOTE;
				this->sixteenthNoteIndex = (this->sixteenthNoteIndex + 1) & 3;    // ... % 4
				this->timeCounter->stop();
				this->timeCounter->start();
			}
		}
		else if ((noChannelByte < 0xA0) && ((this->sixteenthNoteIndex == 1) || (this->sixteenthNoteIndex == 3)) && !this->byPass) {
			DelayedMIDIByte d(this->getDelayedInstant(this->timeCounter->getValue()), byte);
			this->delayQueue.push(d);
			send = false;
		}
		else if (byte == 0xFC)
			this->status = STATUS_WAIT_START_MIDI_CLOCK;
	}
	if (send && (this->sender != NULL))
		this->sender->sendByte(byte);
}


void MIDIShuffler::init(MIDISender &sender, PercentProvider &percentProvider, TimeCounter &timeCounter) {
	MIDIFilter::init(sender);
	this->percentProvider = &percentProvider;
	this->delayQueue.clear();
	this->rxQueue.clear();
	this->status = STATUS_WAIT_START_MIDI_CLOCK;
	this->timeCounter = &timeCounter;
	this->byPass = false;
	this->sixteenthNoteIndex = 0;
}


void MIDIShuffler::run() {
	if (this->rxQueue.hasElements()) {
		uint8_t byte = this->rxQueue.getHead();
		this->processRxByte(byte);
		this->rxQueue.pop();
	}
	if (((this->sixteenthNoteIndex == 1) || (this->sixteenthNoteIndex == 3)) && this->delayQueue.hasElements()) {
		int32_t t = this->timeCounter->getValue();
		while (this->delayQueue.hasElements()) {
			DelayedMIDIByte d = this->delayQueue.getHead();
			if (d.t <= t) {
				this->delayQueue.pop();
				if (this->sender != NULL)
					this->sender->sendByte(d.byte);
			}
			else
				break;
		}
	}
}


void MIDIShuffler::setByPass(bool byPass) {
	this->byPass = byPass;
}

